import unittest
from django_admin_search.meta import VERSION


class TestMetaFormat(unittest.TestCase):

    def test_version_format(self):
        """
            This project use format major.minor.patch
        """
        ver = str(VERSION).split('.')
        self.assertEqual(isinstance(ver, list), True)
        self.assertEqual(len(ver), 3)
        self.assertEqual(all([x.isnumeric() for x in ver]), True)


from django.forms import (CharField, ChoiceField, DateField, Form, ModelForm,
                          TextInput, BooleanField, DateTimeField, TextInput,
                          FloatField, DecimalField, IntegerField)


class TestSearchForm(Form):
    # TODO: Create a field for ModelChoiceField
    STATUS_CHOICES = [
        ('', 'All'),
        (True, 'Active'),
        (False, 'Inactive')
    ]

    field1 = CharField()
    field2 = TextInput()
    field3 = ChoiceField(choices=STATUS_CHOICES)
    field4 = BooleanField()
    field5 = DateField()
    field6 = DateTimeField()
    field7 = FloatField()
    field8 = DecimalField()
    field9 = IntegerField()

    field90 = CharField()
    field91 = CharField()

    def __init__(self, *args, **kwargs):
        super(TestSearchForm, self).__init__(*args, **kwargs)
        self.fields['field90'].label = "Date Start"
        self.fields['field91'].label = "Date End"



import unittest
from django.db.models import Q
from django.contrib import messages
from django_admin_search.admin import AdvancedSearchAdmin
from django_mock_queries.query import MockSet, MockModel
from unittest.mock import patch
from django.contrib.admin.sites import AdminSite
from django.http.request import HttpRequest
from django.contrib.messages.storage.fallback import FallbackStorage


class TestAdminOverride(unittest.TestCase):
    qs = MockSet(MockModel(pk=0, name='test1'), MockModel(pk=1, name='test2'), 
                 MockModel(pk=2, name='test3'))
    
    @patch.object(AdvancedSearchAdmin, 'advanced_search_query')
    @patch('django_admin_search.admin.super')
    def test_get_queryset(self, admin_super, mock_advanced_search_query):
        admin_super().get_queryset.return_value = self.qs
        mock_advanced_search_query.return_value = ~Q(pk=99)

        admin = AdvancedSearchAdmin(model=MockModel, admin_site=AdminSite())

        result = admin.get_queryset({})
        self.assertEqual(result.count(), self.qs.count())

    @patch.object(AdvancedSearchAdmin, 'advanced_search_query')
    @patch('django_admin_search.admin.super')
    def test_get_queryset_by_pk(self, admin_super, mock_advanced_search_query):
        admin_super().get_queryset.return_value = self.qs
        mock_advanced_search_query.return_value = ~Q(pk=2)

        admin = AdvancedSearchAdmin(model=MockModel, admin_site=AdminSite())

        result = admin.get_queryset({})
        self.assertEqual(result.count(), 2)

    @patch.object(AdvancedSearchAdmin, 'advanced_search_query')
    @patch('django_admin_search.admin.super')
    def test_get_queryset_erro_message(self, admin_super, mock_advanced_search_query):
        admin_super().get_queryset.return_value = self.qs
        mock_advanced_search_query.side_effect = Exception('example')

        admin = AdvancedSearchAdmin(model=MockModel, admin_site=AdminSite())
        request = HttpRequest()
        setattr(request, 'session', 'session')
        setattr(request, '_messages', FallbackStorage(request))

        self.assertEqual(admin.get_queryset(request).count(), 0)

        msgs = list(messages.get_messages(request))
        self.assertEqual(len(msgs), 1)
        self.assertEqual(msgs[0].level_tag, 'error')
        self.assertEqual(msgs[0].message, 'Filter not applied, error has occurred')


# -*- coding: utf-8 -*-
from django.contrib import messages
from django.contrib.admin import ModelAdmin
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _
from django.core.exceptions import ValidationError

from django_admin_search import utils


class AdvancedSearchAdmin(ModelAdmin):
    """
        class to add custom filters in django admin
    """
    change_list_template = 'admin/custom_change_list.html'
    advanced_search_fields = {}
    search_form_data = None

    def get_queryset(self, request):
        """
            override django admin 'get_queryset'
        """
        queryset = super().get_queryset(request)
        try:
            return queryset.filter(
                self.advanced_search_query(request)
            )
        except Exception:
            messages.add_message(request, messages.ERROR, 'Filter not applied, error has occurred')
            return queryset.none()

    def changelist_view(self, request, extra_context=None):
        self.search_form_data = self.search_form(request.GET)
        self.extract_advanced_search_terms(request.GET)
        extra_context = {'asf': self.search_form_data}
        return super().changelist_view(request, extra_context=extra_context)

    def extract_advanced_search_terms(self, request):
        request._mutable = True  # pylint: disable=W0212

        if self.search_form_data is not None:
            for key in self.search_form_data.fields.keys():
                temp = request.pop(key, None)
                if temp:  # there is a field but it's empty so it's useless
                    self.advanced_search_fields[key] = temp

        request._mutable = False  # pylint: disable=W0212

    def advanced_search_query(self, request):
        """
            Get form and mount filter query if form is not none
        """
        query = Q()
        param_values = self.advanced_search_fields

        form = self.search_form_data
        if form is None:
            return query

        for field, form_field in self.search_form_data.fields.items():
            field_value = param_values[field][0] if field in param_values else None

            # to overide default filter for a sigle field
            if hasattr(self, ('search_' + field)):
                query &= getattr(self, 'search_' + field)(field, field_value,
                                                          form_field, request,
                                                          param_values)
                continue

            if field_value in [None, '']:
                continue

            field_name = form_field.widget.attrs.get('filter_field', field)
            field_filter = field_name + form_field.widget.attrs.get('filter_method', '')

            try:
                field_value = utils.format_data(form_field, field_value)  # format by field type
                query &= Q(**{field_filter: field_value})
            except ValidationError:
                messages.add_message(request, messages.ERROR, _(f"Filter in field `{field_value}` "
                                                                "ignored, because value "
                                                                "`{field_name}` isn't valid"))
                continue
            except Exception:
                messages.add_message(request, messages.ERROR, _(f"Filter in field `{field_name}` "
                                                                "ignored, error has occurred."))
                continue

        return query



from django.test import SimpleTestCase
from django.template import Context, Template
from django.template.loader import render_to_string

from django_admin_search.templatetags.advanced_search_form import advanced_search_form
from test.mocks import CLMock, TestSearchForm


class TestAdvancedSearchTemplateTag(SimpleTestCase):

    def test_templatag_render(self):
        form = TestSearchForm()
        context = Context({'cl': CLMock, 'asf': form})
        template = Template(
            '{% load advanced_search_form %}'
            '{% advanced_search_form cl %}'
        ).render(context)

        for field, _ in form.fields.items():
            # search fields in template
            self.assertIn(str(form[field]), template)
            # search fields label's in template
            self.assertIn(form[field].label_tag(), template)

    def test_templatag_render_without_asf(self):
        context = Context({'cl': CLMock})
        template = Template(
            '{% load advanced_search_form %}'
            '{% advanced_search_form cl %}'
        ).render(context)
        self.assertEqual('', template)